//+-------------------------------------------------------------------------+
//| t3_adx_+di_-di_burst.mq4                                                |
//| Copyright © 2005,                                                       |
//| http://finance.groups.yahoo.com/group/MetaTrader_Experts_and_Indicators/|
//+-------------------------------------------------------------------------+
#property copyright "Copyright © 2005,"
#property link      "http://finance.groups.yahoo.com/group/MetaTrader_Experts_and_Indicators/"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_label1  "Dmi slope strong up"
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  clrLawnGreen
#property indicator_width1  2

#property indicator_label2  "Dmi slope weak up"
#property indicator_type2   DRAW_HISTOGRAM
#property indicator_color2  clrLawnGreen
#property indicator_width2  1

#property indicator_label3  "Dmi slope strong down"
#property indicator_type3   DRAW_HISTOGRAM
#property indicator_color3  clrRed
#property indicator_width3  2

#property indicator_label4  "Dmi slope weak down"
#property indicator_type4   DRAW_HISTOGRAM
#property indicator_color4  clrRed
#property indicator_width4  1
#property strict


input int                adxPeriods      = 21;              // Adx period
input double             T3Period        = 21;              // T3 smoothing period
input double             T3Hot           = 0.63;            // T3 smoothing hot
enum  enT3Type
      {
        t3_tillson  = (int)true,                            // Tim Tillson way of calculation
        t3_fulksmat = int(false),                           // Fulks/Matulich way of calculation
      };
input enT3Type           T3Original      = t3_tillson;      // T3 calculation mode

double huu[],hud[],hdd[],hdu[],val[],map[],valc[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int OnInit()
{
    IndicatorBuffers(7);
    SetIndexBuffer(0, huu,INDICATOR_DATA); 
    SetIndexBuffer(1, hud,INDICATOR_DATA); 
    SetIndexBuffer(2, hdd,INDICATOR_DATA);
    SetIndexBuffer(3, hdu,INDICATOR_DATA);
    SetIndexBuffer(4, val);
    SetIndexBuffer(5, map); 
    SetIndexBuffer(6, valc); 
    IndicatorSetString(INDICATOR_SHORTNAME," Dmi slope "+(string)T3Period);
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) {}

//------------------------------------------------------------------
// Custom indicator iteration function
//------------------------------------------------------------------

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int i=rates_total-prev_calculated+1; if (i>=rates_total) i=rates_total-1;
   for (; i>=0 && !_StopFlag; i--)
   {
      map[i] = iT3(iADX(_Symbol,_Period,adxPeriods,PRICE_CLOSE,MODE_PLUSDI,i)-iADX(_Symbol,_Period,adxPeriods,PRICE_CLOSE,MODE_MINUSDI,i),T3Period,T3Hot,T3Original,i,rates_total);
      val[i] = map[i]-map[(int)fmin(rates_total-1,i+1)];
      valc[i] = (i<rates_total-1) ? (val[i]>0) ? (val[i]>val[i+1]) ? 0 : 1 : (val[i]<val[i+1]) ? 2 : 3 : 0;
      huu[i] = (valc[i] == 0) ? val[i] : EMPTY_VALUE;
      hud[i] = (valc[i] == 1) ? val[i] : EMPTY_VALUE;
      hdd[i] = (valc[i] == 2) ? val[i] : EMPTY_VALUE;
      hdu[i] = (valc[i] == 3) ? val[i] : EMPTY_VALUE;    
   }   
return(rates_total);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------

#define t3Instances 1
double workT3[][t3Instances*6];
double workT3Coeffs[][6];
#define _tperiod 0
#define _c1      1
#define _c2      2
#define _c3      3
#define _c4      4
#define _alpha   5

//
//
//

double iT3(double price, double period, double hot, bool original, int i, int bars, int tinstanceNo=0)
{
   if (ArrayRange(workT3,0) != bars)                 ArrayResize(workT3,bars);
   if (ArrayRange(workT3Coeffs,0) < (tinstanceNo+1)) ArrayResize(workT3Coeffs,tinstanceNo+1);

   if (workT3Coeffs[tinstanceNo][_tperiod] != period)
   {
     workT3Coeffs[tinstanceNo][_tperiod] = period;
        double a = hot;
            workT3Coeffs[tinstanceNo][_c1]    = -a*a*a;
            workT3Coeffs[tinstanceNo][_c2]    = 3*a*a+3*a*a*a;
            workT3Coeffs[tinstanceNo][_c3]    = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[tinstanceNo][_c4]    = 1+3*a+a*a*a+3*a*a;
            workT3Coeffs[tinstanceNo][_alpha] = (original) ? 2.0/(1.0 + period) : 2.0/(2.0 + (period-1.0)/2.0);
            
   }
   
   //
   //
   //
   
   int instanceNo = tinstanceNo*6;
   int r = bars-i-1;
   if (r == 0)
      {
         workT3[r][0+instanceNo] = price;
         workT3[r][1+instanceNo] = price;
         workT3[r][2+instanceNo] = price;
         workT3[r][3+instanceNo] = price;
         workT3[r][4+instanceNo] = price;
         workT3[r][5+instanceNo] = price;
      }
   else
      {
         workT3[r][0+instanceNo] = workT3[r-1][0+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(price                  -workT3[r-1][0+instanceNo]);
         workT3[r][1+instanceNo] = workT3[r-1][1+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(workT3[r][0+instanceNo]-workT3[r-1][1+instanceNo]);
         workT3[r][2+instanceNo] = workT3[r-1][2+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(workT3[r][1+instanceNo]-workT3[r-1][2+instanceNo]);
         workT3[r][3+instanceNo] = workT3[r-1][3+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(workT3[r][2+instanceNo]-workT3[r-1][3+instanceNo]);
         workT3[r][4+instanceNo] = workT3[r-1][4+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(workT3[r][3+instanceNo]-workT3[r-1][4+instanceNo]);
         workT3[r][5+instanceNo] = workT3[r-1][5+instanceNo]+workT3Coeffs[tinstanceNo][_alpha]*(workT3[r][4+instanceNo]-workT3[r-1][5+instanceNo]);
      }

   //
   //
   //
   
   return(workT3Coeffs[tinstanceNo][_c1]*workT3[r][5+instanceNo] + 
          workT3Coeffs[tinstanceNo][_c2]*workT3[r][4+instanceNo] + 
          workT3Coeffs[tinstanceNo][_c3]*workT3[r][3+instanceNo] + 
          workT3Coeffs[tinstanceNo][_c4]*workT3[r][2+instanceNo]);
}
  
